<?php

class Errors {

    const NoIntegrationMethodSelected  = "No integration method selected";
    const NoTransactionMethodSelected  = "No transaction method selected";
    const NoTransactionTypeSelected    = "No transaction type selected";
    const NoIntegrationSourceSpecified = "No transaction source specified";

    #
    const NoDomainSpecified = "No gateway domain specified";

    # Merchat Details
    const NoMerchantIDSpecified = "No merchant ID specified";
    const NoPasswordSpecified   = "No password specified";

    # Multiple Transaction Types
    const NoAmountSpecified        = "No amount specified";
    const NoCurrencySpecified      = "No transaction currency specified";
    Const InvalidCurrencySpecified = "Invalid currency specified";

    # Card Details Transaction
    const NoCardNumberSpecified = "No card number specified";
    const NoCardCV2Specified    = "No CV2 specified";
    const NoCardExpiryDate      = "No card expiry date specified";
    const InvalidCardExpiryDate = "Invalid card expiry date specified";

    # Cross Reference Transaction
    const NoCrossReferenceSpecified = "No cross reference specified for a cross reference transaction";

    # ThreeDSecure Transaction
    const NoMDSpecified    = "No MD specified for a ThreeDSecure transaction";
    const NoPaRESSpecified = "No PaRES specified for a ThreeDSecure transaction";

    # Hosted Payment Form
    const NoHostedFormMethodSelected           = "No HostedFormMethod type selected";
    const NoReturnTypeSelected                 = "No return type select for a Hosted Payment Form integration";
    const NoHashMethodSelected                 = "No hash method select for a Hosted Payment Form integration";
    const NoPreSharedKeySpecified              = "No PreSharedKey specified";
    const NoResultReturnTypeSelected           = "No server result return type selected";
    const NoResultDeliveryMethodSelected       = "No Result Delivery Method selected";
    const NoCallbackURLSpecified               = "No CallbackURL specified";
    const NoHostedTransactionResponseSpecified = "No hosted transaction response specified";
    # No Communication
    const NoCommunicationWithGateway           = "Couldn't communicate with payment gateway";

}

class IntegrationMethod {

    const NONE                = NULL;
    const DirectAPI           = "Direct API";
    const HostedPaymentForm   = "Hosted Payment Form";
    const TransparentRedirect = "Transparent Redirect";

}

class TransactionMethod {

    const NONE                      = NULL;
    const CardDetailsTransaction    = "CardDetailsTransaction";
    const CrossReferenceTransaction = "CrossReferenceTransaction";
    const ThreeDSecureTransaction   = "ThreeDSecureTransaction";

}

class TransactionType {

    const NONE       = NULL;
    const Sale       = "SALE";
    const PreAuth    = "PREAUTH";
    const Refund     = "REFUND";
    const Collection = "COLLECTION";

}

class AVSPolicy {

    const NONE                    = NULL;
    const FailIfEitherFail        = "E";
    const FailOnlyIfBothFail      = "B";
    const FailOnlyIfAddressFails  = "A";
    const FailOnlyIfPostCodeFails = "P";
    const DoNotFail               = "N";

}

class AVSPartialAddress {

    const NONE = NULL;
    const PASS = "P";
    const FAIL = "F";

}

class AVSPartialPostCode {

    const NONE = NULL;
    const PASS = "P";
    const FAIL = "F";

}

class AVSResultsUnknown {

    const NONE = NULL;
    const PASS = "P";
    const FAIL = "F";

}

class CV2OverridePolicy {

    const NONE = NULL;
    const PASS = "P";
    const FAIL = "F";

}

class CV2ResultsUnknown {

    const NONE = NULL;
    const PASS = "P";
    const FAIL = "F";

}

class HashMethod {

    const NONE     = NULL;
    const SHA1     = "SHA1";
    const MD5      = "MD5";
    const HMACSHA1 = "HMACSHA1";
    const HMACMD5  = "HMACMD5";

}

class ResultDeliveryMethod {

    const NONE        = "NONE";
    const POST        = "POST";
    const SERVER      = "SERVER";
    const SERVER_PULL = "SERVER_PULL";

}

class HostedFormReturnType {

    const NONE            = NULL;
    const HTMLFormCode    = "HTMLFormCode";
    const InstantRedirect = "InstantRedirect";
    const RedirectURL     = "RedirectURL";

}

class HostedFormMethod {

    const NONE           = NULL;
    const Process        = "Process";
    const ReceiveResults = "Receive Results";
    const DisplayResults = "Display Results";

}

abstract class IridiumBase {

    protected $boDebugMode;
    protected $szDebugEmail;
    #
    protected $boDatabaseSupport;
    #
    protected $szPaymentProcessorFullDomain;
    protected $nEntryPointsValidilityTime;
    #
    protected $szTransactionDateTime;
    #
    protected $imIntegrationMethod;
    protected $tmTransactionMethod;
    protected $ttTransactionType;
    protected $szIntegrationSource;
    #
    protected $szAVSOverridePolicy;
    protected $szCV2OverridePolicy;
    protected $boThreeDSecureOverridePolicy;
    #
    protected $boEchoCardType;
    protected $boEchoAmountReceived;
    protected $boEchoAVSCheckResult;
    protected $boEchoCV2CheckResult;
    protected $boEchoThreeDSecureAuthenticationCheckResult;
    #
    protected $nDeviceCategory;
    protected $szAcceptHeaders;
    protected $szUserAgent;
    protected $szCustomerIPAddress;
    #
    protected $szMerchantID;
    protected $szPassword;
    #
    protected $nAmount;
    protected $nAmountUndecimalised;
    #
    protected $icISOCurrency;
    protected $szCurrencyShort3;
    protected $iccISOCurrencyCode;
    protected $nExponent;
    #
    protected $szOrderID;
    protected $szOrderDescription;
    #
    protected $szAddress1;
    protected $szAddress2;
    protected $szAddress3;
    protected $szAddress4;
    protected $szCity;
    protected $szState;
    protected $szPostCode;
    #
    protected $icISOCountry;
    protected $szCountryShort2;
    protected $szCountryShort3;
    protected $iccISOCountryCode;
    #
    protected $szEmailAddress;
    protected $szPhoneNumber;
    #
    protected $szCardName;
    protected $szCardNumber;
    protected $szCardLastFour;
    protected $szCardExpiryDateMonth;
    protected $szCardExpiryDateYear;
    protected $szCardStartDateMonth;
    protected $szCardStartDateYear;
    protected $szCardIssueNumber;
    protected $szCardCV2;
    #
    protected $szOriginCrossReference;
    #
    protected $boTransactionProcessed;
    protected $boTransactionResultsValidated;
    #
    protected $toTransactionObject;
    protected $trTransactionResult;
    protected $todTransactionOutputData;
    #
    protected $boErrorActive;
    protected $szErrorMessage;
    #
    protected $szMD;
    protected $szPaRES;

    public function __construct($szIntegrationSource) {

        require_once ("Config.php");
        require_once ("Hosted/PaymentFormHelper.php");
        require_once ("ThePaymentGateway/PaymentSystem.php");

        $this->szErrorMessage = array();

        if (!$szIntegrationSource == NULL) {
            $this->szIntegrationSource = "[" . $szIntegrationSource . "] ";
        }

        $this->boDebugMode                = FALSE;
        $this->szDebugEmail               = NULL;
        $this->boDatabaseSupport          = FALSE;
        $this->nEntryPointsValidilityTime = 10;

        $this->imIntegrationMethod = IntegrationMethod::NONE;
        $this->tmTransactionMethod = TransactionMethod::NONE;
        $this->ttTransactionType   = TransactionType::NONE;

        $this->boEchoAVSCheckResult                        = true;
        $this->boEchoCV2CheckResult                        = true;
        $this->boEchoThreeDSecureAuthenticationCheckResult = true;
        $this->boEchoCardType                              = true;
        $this->boEchoAmountReceived                        = true;

        #
        $this->nDeviceCategory = 0;
        $this->szAcceptHeaders = "*/*";
        $this->szUserAgent     = $_SERVER["HTTP_USER_AGENT"];

        $this->szCustomerIPAddress = $_SERVER["REMOTE_ADDR"];

        $this->setAVSOverridePolicy(AVSPolicy::NONE, AVSPartialAddress::NONE, AVSPartialPostCode::NONE, AVSResultsUnknown::NONE);
        $this->setCV2OverridePolicy(CV2OverridePolicy::NONE, CV2ResultsUnknown::NONE);
    }

    public static function boolToString($boBool) {
        $szReturnValue = "false";

        if ($boBool) {
            $szReturnValue = "true";
        }

        return ($szReturnValue);
    }

    public static function stringToBool($szString) {
        $boReturnValue = false;

        if (strToUpper($szString) == "TRUE") {
            $boReturnValue = true;
        }

        return ($boReturnValue);
    }

    /**
     * Clean/Sanitise user input
     *
     * @param string $data - the string to clean
     * @return string
     *
     */
    public static function CleanInput($data) {

        $data = trim($data);
        //$data = htmlspecialchars($data);
        //$data = $this -> db -> real_escape_string($data);

        return $data;
    }

    public function setDebugMode($boDebugMode) {
        $this->boDebugMode = $boDebugMode;
    }

    public function getDebugMode() {
        return $this->boDebugMode;
    }

    public function setDebugEmailAddress($szDebugEmailAddress) {
        $this->szDebugEmail = $szDebugEmailAddress;
    }
    public function getDebugEmailAddress(){
        return $this->szDebugEmail;
    }
    public function setDatabaseSupport($boDatabaseSupport) {
        $this->boDatabaseSupport = $boDatabaseSupport;
    }

    public function setPaymentProcessorFullDomain($szPaymentProcessorFullDomain) {
        $this->szPaymentProcessorFullDomain = self::CleanInput($szPaymentProcessorFullDomain);
    }

    public function setEntryPointsValidilityTimeInMinutes($nTimeInMinutes) {
        $this->nEntryPointsValidilityTime = $nTimeInMinutes;
    }
    public function getEntryPointsValidilityTimeInMinutes() {
        return $this->nEntryPointsValidilityTime;
    }

    public function setTransactionDateTime($szDateTime) {
        $this->szTransactionDateTime = self::CleanInput($szDateTime);
    }
    public function getTransactionDateTime() {
        return $this->szTransactionDateTime;
    }

    public function getIntegrationMethod() {
        return $this->imIntegrationMethod;
    }

    public function setTransactionMethod($tmTransactionMethod) {
        $this->tmTransactionMethod = self::CleanInput($tmTransactionMethod);
    }

    public function getTransactionMethod() {
        return $this->tmTransactionMethod;
    }

    public function setTransactionType($ttTransactionType) {
        $this->ttTransactionType = self::CleanInput($ttTransactionType);
    }

    public function getTransactionType() {
        return $this->ttTransactionType;
    }

    public function setIntegrationSource($szIntegrationSource) {
        $this->szIntegrationSource = self::CleanInput($szIntegrationSource);
    }

    public function getIntegrationSource() {
        return $this->szIntegrationSource;
    }

    public function setHandleTransactionResultsFunctionName($HandleTransactionResultsFunctionName) {
        $this->szHandleTransactionResultsFunctionName = $HandleTransactionResultsFunctionName;
    }

    public function setHandleTransactionResultsIncludePath($HandleTransactionResultsIncludePath) {
        $this->szHandleTransactionResultsIncludePath = $HandleTransactionResultsIncludePath;
    }

    public function setEchoCardType($boEchoCardType) {
        $this->boEchoCardType = (bool) self::CleanInput($boEchoCardType);
    }

    public function setEchoAmountReceived($boEchoAmountReceived) {
        $this->boEchoAmountReceived = (bool) self::CleanInput($boEchoAmountReceived);
    }

    public function setEchoAVSCheckResult($boEchoAVSCheckResult) {
        $this->boEchoAVSCheckResult = (bool) self::CleanInput($boEchoAVSCheckResult);
    }

    public function setEchoCV2CheckResult($boEchoCV2CheckResult) {
        $this->boEchoCV2CheckResult = (bool) self::CleanInput($boEchoCV2CheckResult);
    }

    public function setAVSOverridePolicy($AVSPolicyOverride, $AVSPartialAddress, $AVSPartialPostCode, $AVSResultsUnknown) {
        $this->szAVSOverridePolicy = $AVSPolicyOverride . $AVSPartialAddress . $AVSPartialPostCode . $AVSResultsUnknown;
    }

    public function setCV2OverridePolicy($CV2OverridePolicy, $CV2ResultsUnknown) {
        $this->szCV2OverridePolicy = $CV2OverridePolicy . $CV2ResultsUnknown;
    }

    public function setThreeDSecureOverridePolicy($boThreeDSecureOverridePolicy) {
        $this->boThreeDSecureOverridePolicy = (boolean) self::CleanInput($boThreeDSecureOverridePolicy);
    }

    public function setMerchantID($szMerchantID) {
        $this->szMerchantID = self::CleanInput($szMerchantID);
    }

    public function getMerchantID() {
        return $this->szMerchantID;
    }

    public function setPassword($szPassword) {
        $this->szPassword = self::CleanInput($szPassword);
    }

    public function getPassword() {
        return $this->szPassword;
    }

    public function setTransactionAmountAndCurrency($nAmountWithDecimals, $vCurrency) {

        require ("ISOCurrencies.php");

        $vCurrency = self::CleanInput($vCurrency);

        $iclISOCurrencyList->getISOCurrency($vCurrency, $this->icISOCurrency);

        if (!$this->icISOCurrency == NULL) {

            $this->iccISOCurrencyCode = $this->icISOCurrency->getISOCode();
            $this->szCurrencyShort3   = $this->icISOCurrency->getCurrencyShort();
            $this->nExponent          = $this->icISOCurrency->getExponent();

            $nAmountWithDecimals = self::CleanInput($nAmountWithDecimals);

            if (is_numeric($nAmountWithDecimals)) {

                $this->nAmount = $nAmountWithDecimals;

                $this->nAmountUndecimalised = $this->getAmount($this->nAmount, $this->nExponent, FALSE);
            }
        } else {
            $this->boErrorActive  = true;
            $this->szErrorMessage = "Invalid currency supplied [" . $vCurrency . "]";
            if ($this->boDebugMode && $this->szDebugEmail != NULL) {
                mail($this->szDebugEmail, "FUNC: IridiumBase:setResultsAmountAndCurrency()", "ERROR: " . print_r($this->szErrorMessage, 1));
            }
        }
    }

    public function setResultsAmountAndCurrency($nAmountWithoutDecimals, $vCurrency) {

        require ("ISOCurrencies.php");

        $vCurrency = self::CleanInput($vCurrency);

        $iclISOCurrencyList->getISOCurrency($vCurrency, $this->icISOCurrency);

        if (!$this->icISOCurrency == NULL) {
            $this->iccISOCurrencyCode = $this->icISOCurrency->getISOCode();
            $this->szCurrencyShort3   = $this->icISOCurrency->getCurrencyShort();
            $this->nExponent          = $this->icISOCurrency->getExponent();

            $nAmountWithoutDecimals = self::CleanInput($nAmountWithoutDecimals);

            if (is_numeric($nAmountWithoutDecimals)) {

                $this->nAmountUndecimalised = $nAmountWithoutDecimals;

                $this->nAmount = number_format($this->getAmount($this->nAmountUndecimalised, $this->nExponent, TRUE), $this->nExponent);
            }
        } else {
            $this->boErrorActive  = true;
            $this->szErrorMessage = "Invalid currency supplied [" . $vCurrency . "]";
            if ($this->boDebugMode && $this->szDebugEmail != NULL) {
                mail($this->szDebugEmail, "FUNC: IridiumBase:setResultsAmountAndCurrency()", "ERROR: " . print_r($this->szErrorMessage, 1));
            }
        }
    }

    public function getTransactionCurrencyISOCode() {
        return $this->iccISOCurrencyCode;
    }

    public function setOrderID($szOrderID) {
        $this->szOrderID = self::CleanInput($szOrderID);
    }

    public function getOrderID() {
        return $this->szOrderID;
    }

    public function setOrderDescription($szOrderDescription) {
        $this->szOrderDescription = self::CleanInput($szOrderDescription);
    }

    public function getOrderDescription() {
        return $this->szOrderDescription;
    }

    public function setAddress1($szAddress1) {
        $this->szAddress1 = self::CleanInput($szAddress1);
    }

    public function setAddress2($szAddress2) {
        $this->szAddress2 = self::CleanInput($szAddress2);
    }

    public function setAddress3($szAddress3) {
        $this->szAddress3 = self::CleanInput($szAddress3);
    }

    public function setAddress4($szAddress4) {
        $this->szAddress4 = self::CleanInput($szAddress4);
    }

    public function setCity($szCity) {
        $this->szCity = self::CleanInput($szCity);
    }

    public function setState($szState) {
        $this->szState = self::CleanInput($szState);
    }

    public function setPostCode($szPostCode) {
        $this->szPostCode = self::CleanInput($szPostCode);
    }

    public function setCountry($vCountry) {

        $vCountry = self::CleanInput($vCountry);

        require ("ISOCountries.php");

        $iclISOCountryList->getISOCountry($vCountry, $this->icISOCountry);

        if (!is_null($this->icISOCountry)) {
            $this->iccISOCountryCode = $this->icISOCountry->getISOCode();
            $this->szCountryShort2   = $this->icISOCountry->getCountryShort2();
            $this->szCountryShort3   = $this->icISOCountry->getCountryShort3();
        }
    }

    public function setEmailAddress($szEmailAddress) {
        $this->szEmailAddress = self::CleanInput($szEmailAddress);
    }

    public function setPhoneNumber($szPhoneNumber) {
        $this->szPhoneNumber = self::CleanInput($szPhoneNumber);
    }

    public function setCardName($szCardName) {
        $this->szCardName = self::CleanInput($szCardName);
    }

    public function setCardNumber($szCardNumber) {
        $szCardNumber         = self::CleanInput($szCardNumber);
        $this->szCardNumber   = $szCardNumber;
        $this->szCardLastFour = substr($szCardNumber, -4, 4);
    }

    public function getCardNumber() {
        return $this->szCardNumber;
    }

    public function setCardLastFour($szCardLastFour) {
        $this->szCardLastFour = self::CleanInput($szCardLastFour);
    }

    public function getCardLastFour() {
        return $this->szCardLastFour;
    }

    public function setCardExpiryDateMonth($szCardExpiryDateMonth) {
        $this->szCardExpiryDateMonth = self::CleanInput($szCardExpiryDateMonth);
    }

    public function setCardExpiryDateYear($szCardExpiryDateYear) {
        $this->szCardExpiryDateYear = self::CleanInput($szCardExpiryDateYear);
    }

    public function setCardStartDateMonth($szCardStartDateMonth) {
        $this->szCardStartDateMonth = self::CleanInput($szCardStartDateMonth);
    }

    public function setCardStartDateYear($szCardStartDateYear) {
        $this->szCardStartDateYear = self::CleanInput($szCardStartDateYear);
    }

    public function setCardIssueNumber($szCardIssueNumber) {
        $this->szCardIssueNumber = self::CleanInput($szCardIssueNumber);
    }

    public function setCardCV2($szCardCV2) {
        $this->szCardCV2 = self::CleanInput($szCardCV2);
    }

    public function setOriginCrossReference($szCrossReference) {
        $this->szOriginCrossReference = self::CleanInput($szCrossReference);
    }

    public function getOriginCrossReference() {
        return $this->szOriginCrossReference;
    }

    public function setMD($szMD) {
        $this->szMD = self::CleanInput($szMD);
    }

    public function setPaRES($szPaRES) {
        $this->szPaRES = self::CleanInput($szPaRES);
    }

    public function getPaRES() {
        return $this->szPaRES;
    }

    public function getTransactionObject() {
        return $this->toTransactionObject;
    }

    public function getTransactionOutputData() {
        return $this->todTransactionOutputData;
    }

    public function getTransactionResult() {
        return $this->trTransactionResult;
    }

    public function TransactionProcessed() {
        return $this->boTransactionProcessed;
    }

    public function getAmountDecimalised() {
        return $this->nAmount;
    }

    public function getAmountUndecimalised() {
        return $this->nAmountUndecimalised;
    }

    public function HasActiveError() {
        return $this->boErrorActive;
    }

    public function getErrorMessage($index = null) {
        if ($index != null)
        {
            return $this->szErrorMessage[$index];   
        } else {
            return $this->szErrorMessage;
        }
    }

    private function getAmount($Amount, $nExponent, $GetDecimalisedAmount) {

        $nAmount = 0;

        $Amount = round($Amount, $nExponent);
        $Power  = pow(10, $nExponent);
        if ($GetDecimalisedAmount) {
            $nAmount = $Amount / $Power;
        } else {
            $nAmount = $Amount * $Power;
        }

        return $nAmount;
    }

    function IsReadyBase() {

        $this->szErrorMessage = array();

        if ($this->imIntegrationMethod == IntegrationMethod::NONE) {
            array_push($this->szErrorMessage, Errors::NoIntegrationMethodSelected);
        }
        if ($this->szIntegrationSource == NULL) {
            array_push($this->szErrorMessage, Errors::NoIntegrationSourceSpecified);
        }
        if ($this->szMerchantID == NULL) {
            array_push($this->szErrorMessage, Errors::NoMerchantIDSpecified);
        }
        if ($this->szPassword == NULL) {
            array_push($this->szErrorMessage, Errors::NoPasswordSpecified);
        }
    }

    public function Process() {
        $this->szTransactionDateTime = date('Y-m-d H:i:s P');
        $this->szOrderDescription    = $this->szIntegrationSource . " " . $this->szOrderDescription;
    }

}

class IridiumDirect extends IridiumBase {

    private $rgeplRequestGatewayEntryPointList;
    private $szGatewayEntryPointToAttemptFirst;
    #
    private $szAddress1Override;
    private $szAddress2Override;
    private $szAddress3Override;
    private $szAddress4Override;
    private $szCityOverride;
    private $szStateOverride;
    private $szPostCodeOverride;

    public function __construct($szIntegrationSource) {

        parent::__construct($szIntegrationSource);

        $this->imIntegrationMethod = IntegrationMethod::DirectAPI;

        $this->boEchoCardType                = TRUE;
        $this->boEchoAmountReceived          = TRUE;
        $this->boEchoAVSCheckResult          = TRUE;
        $this->boEchoCV2CheckResult          = TRUE;
        $this->boEchoThreeDSecureCheckResult = TRUE;
    }
    
    public function setGatewayEntryPointToAttemptFirst($szGatewayEntryPointToAttemptFirst){
        $this->szGatewayEntryPointToAttemptFirst = $szGatewayEntryPointToAttemptFirst;
    }
    public function setAddress1Override($szAddress1) {
        $this->szAddress1Override = $szAddress1;
    }

    public function setAddress2Override($szAddress2) {
        $this->szAddress2Override = $szAddress2;
    }

    public function setAddress3Override($szAddress3) {
        $this->szAddress3Override = $szAddress3;
    }

    public function setAddress4Override($szAddress4) {
        $this->szAddress4Override = $szAddress4;
    }

    public function setCityOverride($szCity) {
        $this->szCityOverride = $szCity;
    }

    public function setStateOverride($szState) {
        $this->szStateOverride = $szState;
    }

    public function setPostCodeOverride($szPostCode) {
        $this->szPostCodeOverride = $szPostCode;
    }

    public function setCountryOverride($szCountry) {
        $this->szCountryOverride = $szCountry;
    }

    private function InitialiseGatewayEntryPointList(){

        $aEntryPoints =
                array(
                    1 => "https://gw1." . $this->szPaymentProcessorFullDomain,
                    2 => "https://gw2." . $this->szPaymentProcessorFullDomain
        );

        if ($this->szGatewayEntryPointToAttemptFirst == null) {
            $this->szGatewayEntryPointToAttemptFirst= $aEntryPoints[1];
        }

        $this->rgeplRequestGatewayEntryPointList = new RequestGatewayEntryPointList();

        $nEntryPoint = 0;
        foreach ($aEntryPoints as $key => $value) {
            if ($value != $this->szGatewayEntryPointToAttemptFirst) {
                $this->rgeplRequestGatewayEntryPointList->add($value, ($nEntryPoint + 1) * 100, 1);
            } else {
                $this->rgeplRequestGatewayEntryPointList->add($value, 100, 1);
            }
            $nEntryPoint++;
        }
    }

    private function ProcessCardDetailsTransaction() {

        $this->InitialiseGatewayEntryPointList();

        $this->toTransactionObject = new CardDetailsTransaction($this->rgeplRequestGatewayEntryPointList);

        $this->toTransactionObject->getMerchantAuthentication()->setMerchantID($this->szMerchantID);
        $this->toTransactionObject->getMerchantAuthentication()->setPassword($this->szPassword);

        $this->toTransactionObject->getTransactionDetails()->getMessageDetails()->setTransactionType($this->ttTransactionType);

        $this->toTransactionObject->getTransactionDetails()->getAmount()->setValue($this->nAmountUndecimalised);

        $this->toTransactionObject->getTransactionDetails()->getCurrencyCode()->setValue($this->iccISOCurrencyCode);

        $this->toTransactionObject->getTransactionDetails()->setOrderID($this->szOrderID);
        $this->toTransactionObject->getTransactionDetails()->setOrderDescription($this->szOrderDescription);

        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getEchoCardType()->setValue($this->boEchoCardType);
        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getEchoAmountReceived()->setValue($this->boEchoAmountReceived);
        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getEchoAVSCheckResult()->setValue($this->boEchoAVSCheckResult);
        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getEchoCV2CheckResult()->setValue($this->boEchoCV2CheckResult);
        //$this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getEchoThreeDSecureCheckResult()->setValue($this->boEchoThreeDSecureCheckResult);
        //if (!SharedFunctions::isStringNullOrEmpty($this->szAVSOverridePolicy)) {
        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->setAVSOverridePolicy($this->szAVSOverridePolicy);
        //}
        //if (!SharedFunctions::isStringNullOrEmpty($this->szCV2OverridePolicy)) {
        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->setCV2OverridePolicy($this->szCV2OverridePolicy);
        //}
        if (!is_null($this->boThreeDSecureOverridePolicy)) {
            $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getThreeDSecureOverridePolicy()->setValue($this->boThreeDSecureOverridePolicy);
        }

        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getDuplicateDelay()->setValue(60);

        $this->toTransactionObject->getTransactionDetails()->getThreeDSecureBrowserDetails()->getDeviceCategory()->setValue($this->nDeviceCategory);
        $this->toTransactionObject->getTransactionDetails()->getThreeDSecureBrowserDetails()->setAcceptHeaders($this->szAcceptHeaders);
        $this->toTransactionObject->getTransactionDetails()->getThreeDSecureBrowserDetails()->setUserAgent($this->szUserAgent);

        $this->toTransactionObject->getCardDetails()->setCardName($this->szCardName);
        $this->toTransactionObject->getCardDetails()->setCardNumber($this->szCardNumber);

        if ($this->szCardExpiryDateMonth != "") {
            $this->toTransactionObject->getCardDetails()->getExpiryDate()->getMonth()->setValue($this->szCardExpiryDateMonth);
        }
        if ($this->szCardExpiryDateYear != "") {
            $this->toTransactionObject->getCardDetails()->getExpiryDate()->getYear()->setValue($this->szCardExpiryDateYear);
        }
        if ($this->szCardStartDateMonth != "") {
            $this->toTransactionObject->getCardDetails()->getStartDate()->getMonth()->setValue($this->szCardStartDateMonth);
        }
        if ($this->szCardStartDateYear != "") {
            $this->toTransactionObject->getCardDetails()->getStartDate()->getYear()->setValue($this->szCardStartDateYear);
        }

        $this->toTransactionObject->getCardDetails()->setIssueNumber($this->szCardIssueNumber);
        $this->toTransactionObject->getCardDetails()->setCV2($this->szCardCV2);

        $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setAddress1($this->szAddress1);
        $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setAddress2($this->szAddress2);
        $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setAddress3($this->szAddress3);
        $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setAddress4($this->szAddress4);
        $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setCity($this->szCity);
        $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setState($this->szState);
        $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setPostCode($this->szPostCode);

        $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->getCountryCode()->setValue($this->iccISOCountryCode);

        $this->toTransactionObject->getCustomerDetails()->setEmailAddress($this->szEmailAddress);
        $this->toTransactionObject->getCustomerDetails()->setPhoneNumber($this->szPhoneNumber);
        $this->toTransactionObject->getCustomerDetails()->setCustomerIPAddress($this->szCustomerIPAddress);

        $this->boTransactionProcessed = $this->toTransactionObject->processTransaction($this->trTransactionResult, $this->todTransactionOutputData);
    }

    private function ProcessCrossReferenceTransaction() {

        $this->InitialiseGatewayEntryPointList();

        $this->toTransactionObject = new CrossReferenceTransaction($this->rgeplRequestGatewayEntryPointList);

        $this->toTransactionObject->getMerchantAuthentication()->setMerchantID($this->szMerchantID);
        $this->toTransactionObject->getMerchantAuthentication()->setPassword($this->szPassword);

        $this->toTransactionObject->getTransactionDetails()->getMessageDetails()->setTransactionType($this->ttTransactionType);
        $this->toTransactionObject->getTransactionDetails()->getMessageDetails()->setCrossReference($this->szOriginCrossReference);

        $this->toTransactionObject->getTransactionDetails()->getAmount()->setValue($this->nAmountUndecimalised);

        $this->toTransactionObject->getTransactionDetails()->getCurrencyCode()->setValue($this->iccISOCurrencyCode);

        $this->toTransactionObject->getTransactionDetails()->setOrderID($this->szOrderID);
        $this->toTransactionObject->getTransactionDetails()->setOrderDescription($this->szOrderDescription);

        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getEchoAmountReceived()->setValue($this->boEchoAmountReceived);
        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getEchoAVSCheckResult()->setValue($this->boEchoAVSCheckResult);
        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getEchoCV2CheckResult()->setValue($this->boEchoCV2CheckResult);

        //if (!SharedFunctions::isStringNullOrEmpty($this->szAVSOverridePolicy)) {
        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->setAVSOverridePolicy($this->szAVSOverridePolicy);
        //}
        //if (!SharedFunctions::isStringNullOrEmpty($this->szCV2OverridePolicy)) {
        $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->setCV2OverridePolicy($this->szCV2OverridePolicy);
        //}
        if (!SharedFunctions::isStringNullOrEmpty($this->boThreeDSecureOverridePolicy)) {
            $this->toTransactionObject->getTransactionDetails()->getTransactionControl()->getThreeDSecureOverridePolicy()->setValue($this->boThreeDSecureOverridePolicy);
        }

        if (!SharedFunctions::isStringNullOrEmpty($this->szAddress1)) {
            $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setAddress1($this->szAddress1);
        }
        if (!SharedFunctions::isStringNullOrEmpty($this->szAddress2)) {
            $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setAddress2($this->szAddress2);
        }
        if (!SharedFunctions::isStringNullOrEmpty($this->szAddress3)) {
            $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setAddress3($this->szAddress3);
        }
        if (!SharedFunctions::isStringNullOrEmpty($this->szAddress4)) {
            $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setAddress4($this->szAddress4);
        }
        if (!SharedFunctions::isStringNullOrEmpty($this->szCity)) {
            $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setCity($this->szCity);
        }
        if (!SharedFunctions::isStringNullOrEmpty($this->szState)) {
            $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setState($this->szState);
        }
        if (!SharedFunctions::isStringNullOrEmpty($this->szPostCode)) {
            $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->setPostCode($this->szPostCode);
        }
        if (!SharedFunctions::isStringNullOrEmpty($this->iccISOCountryCode)) {
            $this->toTransactionObject->getCustomerDetails()->getBillingAddress()->getCountryCode()->setValue($this->iccISOCountryCode);
        }

        if (($this->szCardCV2 != "") && ($this->szCardCV2 != NULL)) {
            $this->toTransactionObject->getOverrideCardDetails()->setCV2($this->szCardCV2);
        }

        $this->boTransactionProcessed = $this->toTransactionObject->processTransaction($this->trTransactionResult, $this->todTransactionOutputData);
    }

    private function ProcessThreeDSecureTransaction() {

        $this->InitialiseGatewayEntryPointList();

        $this->toTransactionObject = new ThreeDSecureAuthentication($this->rgeplRequestGatewayEntryPointList);

        $this->toTransactionObject->getMerchantAuthentication()->setMerchantID($this->szMerchantID);
        $this->toTransactionObject->getMerchantAuthentication()->setPassword($this->szPassword);

        $this->toTransactionObject->getThreeDSecureInputData()->setCrossReference($this->szMD);
        $this->toTransactionObject->getThreeDSecureInputData()->setPaRES($this->szPaRES);

        $this->boTransactionProcessed = $this->toTransactionObject->processTransaction($this->trTransactionResult, $this->todTransactionOutputData);
    }

    private function IsReady() {

        parent::IsReadyBase();

        if ($this->tmTransactionMethod == TransactionMethod::NONE) {
            array_push($this->szErrorMessage, Errors::NoTransactionMethodSelected);
        }
        if ($this->szPaymentProcessorFullDomain == NULL) {
            array_push($this->szErrorMessage, Errors::NoDomainSpecified);
        }

        if ($this->nAmount == NULL) {
            array_push($this->szErrorMessage, Errors::NoAmountSpecified);
        }
        if ($this->iccISOCurrencyCode == NULL) {
            array_push($this->szErrorMessage, Errors::NoCurrencySpecified);
        }
        if (($this->ttTransactionType == TransactionType::NONE) && ($this->tmTransactionMethod != TransactionMethod::ThreeDSecureTransaction)) {
            array_push($this->szErrorMessage, Errors::NoTransactionTypeSelected);
        }

        switch ($this->tmTransactionMethod) {
            case TransactionMethod::CardDetailsTransaction :

                if ($this->szCardNumber == NULL) {
                    array_push($this->szErrorMessage, Errors::NoCardNumberSpecified);
                }
                if ($this->szCardCV2 == NULL) {
                    array_push($this->szErrorMessage, Errors::NoCardCV2Specified);
                }
                if ($this->szCardExpiryDateMonth == NULL && $this->szCardExpiryDateYear == NULL) {
                    array_push($this->szErrorMessage, Errors::NoCardExpiryDate);
                } elseif ($this->szCardExpiryDateMonth == NULL || $this->szCardExpiryDateYear == NULL || $this->szCardExpiryDateMonth < 1 ||
                        $this->szCardExpiryDateMonth > 12) {
                    array_push($this->szErrorMessage, Errors::InvalidCardExpiryDate);
                }
                break;
            case TransactionMethod::CrossReferenceTransaction :
                if ($this->szOriginCrossReference == NULL) {
                    array_push($this->szErrorMessage, Errors::NoCrossReferenceSpecified);
                }
                break;
            case TransactionMethod::ThreeDSecureTransaction :
                if ($this->szMD == NULL) {
                    array_push($this->szErrorMessage, Errors::NoMDSpecified);
                }
                if ($this->szPaRES == NULL) {
                    array_push($this->szErrorMessage, Errors::NoPaRESSpecified);
                }
                break;
        }

        $this->boErrorActive = !(empty($this->szErrorMessage));

        if ($this->boErrorActive && $this->boDebugMode && $this->szDebugEmail != NULL) {
            mail($this->szDebugEmail, "FUNC: IridiumDirect:IsReady()", "ERROR: " . print_r($this->szErrorMessage, 1));
        }

        return !$this->boErrorActive;
    }

    public function Process() {

        $return = NULL;

        if ($this->IsReady()) {

            parent::Process();

            switch ($this->tmTransactionMethod) {
                case TransactionMethod::NONE :
                    break;
                case TransactionMethod::CardDetailsTransaction :
                    $this->ProcessCardDetailsTransaction();
                    break;
                case TransactionMethod::CrossReferenceTransaction :
                    $this->ProcessCrossReferenceTransaction();
                    break;
                case TransactionMethod::ThreeDSecureTransaction :
                    $this->ProcessThreeDSecureTransaction();
                    break;
            }

            if ($return == NULL && !$this->boTransactionProcessed) {
                $this->boErrorActive = TRUE;
                array_push($this->szErrorMessage, Errors::NoCommunicationWithGateway);
            } elseif ($this->boDatabaseSupport) {
                IridiumDatabase::insertGEP_EntryPoint($this->toTransactionObject->getEntryPointUsed(), $this->szTransactionDateTime, $this->nEntryPointsValidilityTime);
            }
        } else {
            $return = FALSE;
        }


        return $return;
    }

}

class IridiumHosted extends IridiumBase {

    #

    private $szHostedCrossReference;
    #
    private $hfrtHostedFormReturnType;
    private $hfmHostedFormMethod;
    #
    private $boEmailAddressEditable;
    private $boPhoneNumberEditable;
    #
    private $boCV2Mandatory;
    private $boAddress1Mandatory;
    private $boCityMandatory;
    private $boPostCodeMandatory;
    private $boStateMandatory;
    private $boCountryMandatory;
    #
    private $boIncludeFormTags;
    private $boIncludeFormSubmitButton;
    #
    private $szPreSharedKey;
    private $boPaymentFormDisplaysResult;
    private $rdmResultDeliveryMethod;
    private $szServerResultURL;
    private $szCallbackURL;
    #
    private $szServerResultURLCookieVariables;
    private $szServerResultURLFormVariables;
    private $szServerResultURLQueryStringVariables;
    #
    private $hmHashMethod;
    private $szStringToHash;
    private $szHashDigest;
    # results
    private $aHostedTransactionResponse;
    private $aHostedResponsePaymentHandler;

    //private $this->trTransactionResult;

    public function __construct($szIntegrationSource) {

        parent::__construct($szIntegrationSource);

        $this->imIntegrationMethod = IntegrationMethod::HostedPaymentForm;

        $this->rdmResultDeliveryMethod = ResultDeliveryMethod::NONE;

        $this->boThreeDSecureOverridePolicy = TRUE;

        $this->boEmailAddressEditable = TRUE;
        $this->boPhoneNumberEditable  = TRUE;

        $this->boIncludeFormTags         = TRUE;
        $this->boIncludeFormSubmitButton = TRUE;
    }

    public function setEmailAddressEditable($boEmailAddressEditable) {
        $this->boEmailAddressEditable = (bool) self::CleanInput($boEmailAddressEditable);
    }

    public function setPhoneNumberEditable($boPhoneNumberEditable) {
        $this->boPhoneNumberEditable = (bool) self::CleanInput($boPhoneNumberEditable);
    }

    public function setHostedFormReturnType($hfrtHostedFormReturnType) {
        $this->hfrtHostedFormReturnType = self::CleanInput($hfrtHostedFormReturnType);
    }

    public function setHostedFormMethod($hfmHostedFormMethod) {
        $this->hfmHostedFormMethod = $hfmHostedFormMethod;
    }
    public function getHostedFormMethod() {
        return $this->hfmHostedFormMethod;
    }

    public function setHTML_IncludeFormTags($boIncludeFormTags) {
        $this->boIncludeFormTags = $boIncludeFormTags;
    }

    public function setHTML_IncludeSubmitButton($boIncludeFormSubmitButton) {
        $this->boIncludeFormSubmitButton = $boIncludeFormSubmitButton;
    }

    public function setHTMLForm_ButtonCaption($szCaption) {
        $this->szHTML_ButtonCaption = self::CleanInput($szCaption);
    }

    public function setCV2Mandatory($boCV2Mandatory) {
        $this->boCV2Mandatory = (bool) self::CleanInput($boCV2Mandatory);
    }

    public function setAddress1Mandatory($boAddress1Mandatory) {
        $this->boAddress1Mandatory = (bool) self::CleanInput($boAddress1Mandatory);
    }

    public function setCityMandatory($boCityMandatory) {
        $this->boCityMandatory = (bool) self::CleanInput($boCityMandatory);
    }

    public function setPostCodeMandatory($boPostCodeMandatory) {
        $this->boPostCodeMandatory = (bool) self::CleanInput($boPostCodeMandatory);
    }

    public function setStateMandatory($boStateMandatory) {
        $this->boStateMandatory = (bool) self::CleanInput($boStateMandatory);
    }

    public function setCountryMandatory($boCountryMandatory) {
        $this->boCountryMandatory = (bool) self::CleanInput($boCountryMandatory);
    }

    public function setPaymentFormDisplayResult($boPaymentFormDisplayResult) {
        $this->boPaymentFormDisplaysResult = (bool) self::CleanInput($boPaymentFormDisplayResult);
    }

    public function getPaymentFormDisplayResult() {
        return $this->boPaymentFormDisplaysResult;
    }

    public function setResultDeliveryMethod($rdmResultDeliveryMethod) {
        $this->rdmResultDeliveryMethod = self::CleanInput($rdmResultDeliveryMethod);
    }

    public function setServerResultURL($szServerResultURL) {
        $szServerResultURL = self::CleanInput($szServerResultURL);
        if ($szServerResultURL != NULL) {
            $this->szServerResultURL = "http://dev.iridiumcorp.net/CubeCart/" . $szServerResultURL;
            //$this->szServerResultURL = $this->getSiteSecureBaseURL() . $szServerResultURL;
        } else {
            $this->szServerResultURL = $szServerResultURL;
        }
    }

    public function setCallbackURL($szCallbackURL) {
        $szCallbackURL = self::CleanInput($szCallbackURL);
        if ($szCallbackURL != NULL) {
            $this->szCallbackURL = $this->getSiteSecureBaseURL() . $szCallbackURL;
            //$this->szCallbackURL = "http://dev.iridiumcorp.net/CubeCart/" . $szCallbackURL;
        } else {
            $this->szCallbackURL = $szCallbackURL;
        }
    }

    public function setHashMethod($hmHashMethod) {
        $this->hmHashMethod = self::CleanInput($hmHashMethod);
    }

    public function getHashMethod() {
        return $this->hmHashMethod;
    }

    public function setPreSharedKey($szPreSharedKey) {
        $this->szPreSharedKey = self::CleanInput($szPreSharedKey);
    }

    public function getPreSharedKey() {
        return $this->szPreSharedKey;
    }

    public function setHostedTransactionResponse($aHostedTransactionResponse) {
        $this->aHostedTransactionResponse = $aHostedTransactionResponse;
    }

    public function getHostedTransactionResponse() {
        return $this->aHostedTransactionResponse;
    }

    public function getHostedPaymentHandler() {
        return "https://mms." . $this->szPaymentProcessorFullDomain . "Pages/PublicPages/PaymentFormResultHandler.ashx";
    }

    private function getTransactionReferenceFromQueryString() {

        $this->szHashDigest   = "";
        $this->szErrorMessage = "";
        $this->boErrorActive  = false;

        try {
            // hash digest
            if (isset($this->aHostedTransactionResponse["HashDigest"])) {
                $this->szHashDigest = $this->aHostedTransactionResponse["HashDigest"];
            }

            // cross reference of transaction
            if (!isset($this->aHostedTransactionResponse["CrossReference"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [CrossReference] not received");
                $this->boErrorActive  = true;
            } else {
                $this->szHostedCrossReference = $this->aHostedTransactionResponse["CrossReference"];
            }
            // order ID (same as value passed into payment form - echoed back out by payment form)
            if (!isset($this->aHostedTransactionResponse["OrderID"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [OrderID] not received");
                $this->boErrorActive  = true;
            } else {
                $this->szOrderID = $this->aHostedTransactionResponse["OrderID"];
            }
        } catch (Exception $e) {
            $this->boErrorActive  = true;
            $this->szErrorMessage = $e->getMessage();
            
            if ($this->boDebugMode && $this->szDebugEmail) {
                mail($this->szDebugEmail, "FUNC: getTransactionReferenceFromQueryString", "SERVER RESPONSE: \n" . print_r($this->aHostedTransactionResponse, 1));
            }
        }
        return (!$this->boErrorActive);
    }

    private function getTransactionResultFromPostVariables() {

        //$this->trTransactionResult = null;
        $this->szHashDigest   = "";
        $this->szErrorMessage = "";
        $this->boErrorActive  = false;

        try {
            $this->trTransactionResult = new TransactionResult();

            // hash digest
            if (isset($this->aHostedTransactionResponse["HashDigest"])) {
                $this->szHashDigest = $this->aHostedTransactionResponse["HashDigest"];
            }

            // transaction status code
            if (!isset($this->aHostedTransactionResponse["StatusCode"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [StatusCode] not received");
                $this->boErrorActive  = true;
            } else {
                if ($this->aHostedTransactionResponse["StatusCode"] == "") {
                    $this->trTransactionResult->setStatusCode(null);
                } else {
                    $this->trTransactionResult->setStatusCode(intval($this->aHostedTransactionResponse["StatusCode"]));
                }
            }
            // transaction message
            if (!isset($this->aHostedTransactionResponse["Message"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [Message] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setMessage($this->aHostedTransactionResponse["Message"]);
            }
            // status code of original transaction if this transaction was deemed a duplicate
            if (!isset($this->aHostedTransactionResponse["PreviousStatusCode"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [PreviousStatusCode] not received");
                $this->boErrorActive  = true;
            } else {
                if ($this->aHostedTransactionResponse["PreviousStatusCode"] == "") {
                    $this->trTransactionResult->setPreviousStatusCode(null);
                } else {
                    $this->trTransactionResult->setPreviousStatusCode(intval($this->aHostedTransactionResponse["PreviousStatusCode"]));
                }
            }
            // status code of original transaction if this transaction was deemed a duplicate
            if (!isset($this->aHostedTransactionResponse["PreviousMessage"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [PreviousMessage] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setPreviousMessage($this->aHostedTransactionResponse["PreviousMessage"]);
            }
            // cross reference of transaction
            if (!isset($this->aHostedTransactionResponse["CrossReference"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [CrossReference] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setCrossReference($this->aHostedTransactionResponse["CrossReference"]);
            }
            // amount (same as value passed into payment form - echoed back out by payment form)
            if (!isset($this->aHostedTransactionResponse["Amount"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [Amount] not received");
                $this->boErrorActive  = true;
            } else {
                if ($this->aHostedTransactionResponse["Amount"] == "") {
                    $this->trTransactionResult->setAmount(NULL);
                } else {
                    $this->trTransactionResult->setAmount($this->aHostedTransactionResponse["Amount"]);
                }
            }

            if (!isset($this->aHostedTransactionResponse["CurrencyCode"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [CurrencyCode] not received");
                $this->boErrorActive  = true;
            } else {
                if ($this->aHostedTransactionResponse["CurrencyCode"] == "") {
                    $this->trTransactionResult->setCurrencyCode(NULL);
                } else {
                    $this->trTransactionResult->setCurrencyCode($this->aHostedTransactionResponse["CurrencyCode"]);
                }
            }
            // currency code (same as value passed into payment form - echoed back out by payment form)
            // order ID (same as value passed into payment form - echoed back out by payment form)
            if (!isset($this->aHostedTransactionResponse["OrderID"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [OrderID] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setOrderID($this->aHostedTransactionResponse["OrderID"]);
            }
            // transaction type (same as value passed into payment form - echoed back out by payment form)
            if (!isset($this->aHostedTransactionResponse["TransactionType"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [TransactionType] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setTransactionType($this->aHostedTransactionResponse["TransactionType"]);
            }
            // transaction date/time (same as value passed into payment form - echoed back out by payment form)
            if (!isset($this->aHostedTransactionResponse["TransactionDateTime"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [TransactionDateTime] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setTransactionDateTime($this->aHostedTransactionResponse["TransactionDateTime"]);
            }
            // order description (same as value passed into payment form - echoed back out by payment form)
            if (!isset($this->aHostedTransactionResponse["OrderDescription"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [OrderDescription] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setOrderDescription($this->aHostedTransactionResponse["OrderDescription"]);
            }
            // customer name (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (!isset($this->aHostedTransactionResponse["CustomerName"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [CustomerName] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setCustomerName($this->aHostedTransactionResponse["CustomerName"]);
            }
            // address1 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (!isset($this->aHostedTransactionResponse["Address1"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [Address1] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setAddress1($this->aHostedTransactionResponse["Address1"]);
            }
            // address2 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (!isset($this->aHostedTransactionResponse["Address2"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [Address2] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setAddress2($this->aHostedTransactionResponse["Address2"]);
            }
            // address3 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (!isset($this->aHostedTransactionResponse["Address3"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [Address3] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setAddress3($this->aHostedTransactionResponse["Address3"]);
            }
            // address4 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (!isset($this->aHostedTransactionResponse["Address4"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [Address4] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setAddress4($this->aHostedTransactionResponse["Address4"]);
            }
            // city (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (!isset($this->aHostedTransactionResponse["City"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [City] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setCity($this->aHostedTransactionResponse["City"]);
            }
            // state (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (!isset($this->aHostedTransactionResponse["State"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [State] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setState($this->aHostedTransactionResponse["State"]);
            }
            // post code (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (!isset($this->aHostedTransactionResponse["PostCode"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [PostCode] not received");
                $this->boErrorActive  = true;
            } else {
                $this->trTransactionResult->setPostCode($this->aHostedTransactionResponse["PostCode"]);
            }
            // country code (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (!isset($this->aHostedTransactionResponse["CountryCode"])) {
                $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [CountryCode]mail not received");
                $this->boErrorActive  = true;
            } else {
                if ($this->aHostedTransactionResponse["CountryCode"] == "") {
                    $this->trTransactionResult->setCountryCode(NULL);
                } else {
                    $this->trTransactionResult->setCountryCode($this->aHostedTransactionResponse["CountryCode"]);
                }
            }

            if ($this->rdmResultDeliveryMethod != ResultDeliveryMethod::SERVER_PULL) {
                if (!isset($this->aHostedTransactionResponse["AddressNumericCheckResult"])) {
                    $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [AddressNumericCheckResult] not received");
                    $this->boErrorActive  = true;
                } else {
                    if ($this->aHostedTransactionResponse["AddressNumericCheckResult"] == "") {
                        $this->trTransactionResult->setAddressNumericCheckResult(null);
                    } else {
                        $this->trTransactionResult->setAddressNumericCheckResult($this->aHostedTransactionResponse["AddressNumericCheckResult"]);
                    }
                }
                if (!isset($this->aHostedTransactionResponse["PostCodeCheckResult"])) {
                    $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [PostCodeCheckResult] not received");
                    $this->boErrorActive  = true;
                } else {
                    if ($this->aHostedTransactionResponse["PostCodeCheckResult"] == "") {
                        $this->trTransactionResult->setPostCodeCheckResult(null);
                    } else {
                        $this->trTransactionResult->setPostCodeCheckResult($this->aHostedTransactionResponse["PostCodeCheckResult"]);
                    }
                }
                if (!isset($this->aHostedTransactionResponse["CV2CheckResult"])) {
                    $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [CV2CheckResult] not received");
                    $this->boErrorActive  = true;
                } else {
                    if ($this->aHostedTransactionResponse["CV2CheckResult"] == "") {
                        $this->trTransactionResult->setCV2CheckResult(null);
                    } else {
                        $this->trTransactionResult->setCV2CheckResult($this->aHostedTransactionResponse["CV2CheckResult"]);
                    }
                }
                if (!isset($this->aHostedTransactionResponse["ThreeDSecureCheckResult"])) {
                    $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [ThreeDSecureCheckResult] not received");
                    $this->boErrorActive  = true;
                } else {
                    if ($this->aHostedTransactionResponse["ThreeDSecureCheckResult"] == "") {
                        $this->trTransactionResult->setThreeDSecureCheckResult(null);
                    } else {
                        $this->trTransactionResult->setThreeDSecureCheckResult($this->aHostedTransactionResponse["ThreeDSecureCheckResult"]);
                    }
                }
                if (!isset($this->aHostedTransactionResponse["CardType"])) {
                    $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [CardType] not received");
                    $this->boErrorActive  = true;
                } else {
                    if ($this->aHostedTransactionResponse["CardType"] == "") {
                        $this->trTransactionResult->setCardType(null);
                    } else {
                        $this->trTransactionResult->setCardType($this->aHostedTransactionResponse["CardType"]);
                    }
                }
                if (!isset($this->aHostedTransactionResponse["CardClass"])) {
                    $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [CardClass] not received");
                    $this->boErrorActive  = true;
                } else {
                    if ($this->aHostedTransactionResponse["CardClass"] == "") {
                        $this->trTransactionResult->setCardClass(null);
                    } else {
                        $this->trTransactionResult->setCardClass($this->aHostedTransactionResponse["CardClass"]);
                    }
                }
                if (!isset($this->aHostedTransactionResponse["CardIssuer"])) {
                    $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [CardIssuer] not received");
                    $this->boErrorActive  = true;
                } else {
                    if ($this->aHostedTransactionResponse["CardIssuer"] == "") {
                        $this->trTransactionResult->setCardIssuer(null);
                    } else {
                        $this->trTransactionResult->setCardIssuer($this->aHostedTransactionResponse["CardIssuer"]);
                    }
                }
                if (!isset($this->aHostedTransactionResponse["CardIssuerCountryCode"])) {
                    $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [CardIssuerCountryCode] not received");
                    $this->boErrorActive  = true;
                } else {
                    if ($this->aHostedTransactionResponse["CardIssuerCountryCode"] == "") {
                        $this->trTransactionResult->setCardIssuerCountryCode(null);
                    } else {
                        $this->trTransactionResult->setCardIssuerCountryCode(intval($this->aHostedTransactionResponse["CardIssuerCountryCode"]));
                    }
                }
                if (!isset($this->aHostedTransactionResponse["EmailAddress"])) {
                    $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [EmailAddress] not received");
                    $this->boErrorActive  = true;
                } else {
                    if ($this->aHostedTransactionResponse["EmailAddress"] == "") {
                        $this->trTransactionResult->setEmailAddress(null);
                    } else {
                        $this->trTransactionResult->setEmailAddress($this->aHostedTransactionResponse["EmailAddress"]);
                    }
                }
                if (!isset($this->aHostedTransactionResponse["PhoneNumber"])) {
                    $this->szErrorMessage = PaymentFormHelper::addStringToStringList($this->szErrorMessage, "Expected variable [PhoneNumber] not received");
                    $this->boErrorActive  = true;
                } else {
                    if ($this->aHostedTransactionResponse["PhoneNumber"] == "") {
                        $this->trTransactionResult->setPhoneNumber(null);
                    } else {
                        $this->trTransactionResult->setPhoneNumber($this->aHostedTransactionResponse["PhoneNumber"]);
                    }
                }
            }

            if ($this->boErrorActive && $this->boDebugMode && $this->szDebugEmail != NULL) {
                mail($this->szDebugEmail, "FUNC: IridiumHosted:getTransactionResultFromPostVariables()", "ERROR: \n" . print_r($this->szErrorMessage, 1));
            }
        } catch (Exception $e) {
            $this->boErrorActive  = true;
            $this->szErrorMessage = $e->getMessage();

            if ($this->boDebugMode && $this->szDebugEmail != NULL) {
                mail($this->szDebugEmail, "FUNC: IridiumHosted:getTransactionResultFromPostVariables()", "ERROR: \n" . print_r($this->szErrorMessage, 1));
            }
        }

        return (!$this->boErrorActive);
    }

    public static function getSiteSecureBaseURL() {

        $szReturnString   = "";
        $szPortString     = "";
        $szProtocolString = "";

        if ($_SERVER["HTTPS"] == "on") {
            $szProtocolString = "https://";
            if ($_SERVER["SERVER_PORT"] != 443) {
                $szPortString = ":" . $_SERVER["SERVER_PORT"];
            }
        } else {
            $szProtocolString = "http://";
            if ($_SERVER["SERVER_PORT"] != 80) {
                $szPortString = ":" . $_SERVER["SERVER_PORT"];
            }
        }

        $szReturnString = $szProtocolString . $_SERVER["SERVER_NAME"] . $szPortString . $_SERVER["SCRIPT_NAME"];

        $boFinished = false;
        $LoopIndex  = strlen($szReturnString) - 1;

        while ($boFinished == false &&
        $LoopIndex >= 0) {
            if ($szReturnString[$LoopIndex] == "/") {
                $boFinished     = true;
                $szReturnString = substr($szReturnString, 0, $LoopIndex + 1);
            }
            $LoopIndex--;
        }

        return ($szReturnString);
    }

    private function generateStringToHash() {

        $szReturnString = "";

        if ($this->IncludePreSharedKeyInString()) {
            $szReturnString = "PreSharedKey=" . $this->szPreSharedKey . "&";
        }

        $szReturnString .=
                "MerchantID=" . $this->szMerchantID .
                "&Password=" . $this->szPassword .
                "&Amount=" . $this->nAmountUndecimalised .
                "&CurrencyCode=" . $this->iccISOCurrencyCode .
                "&EchoAVSCheckResult=" . self::boolToString($this->boEchoAVSCheckResult) .
                "&EchoCV2CheckResult=" . self::boolToString($this->boEchoCV2CheckResult) .
                "&EchoThreeDSecureAuthenticationCheckResult=" . self::boolToString($this->boEchoThreeDSecureAuthenticationCheckResult) .
                "&EchoCardType=" . self::boolToString($this->boEchoCardType);

        //            if (!$szAVSOverridePolicy == NULL) {
        //                $szReturnString .= "&AVSOverridePolicy=".$szAVSOverridePolicy;
        //            }
        //            if (!$szCV2OverridePolicy == NULL) {
        //                $szReturnString .= "&CV2OverridePolicy=".$szCV2OverridePolicy;
        //            }

        $szReturnString .=
                "&ThreeDSecureOverridePolicy=" . self::boolToString($this->boThreeDSecureOverridePolicy) .
                "&OrderID=" . $this->szOrderID .
                "&TransactionType=" . $this->ttTransactionType .
                "&TransactionDateTime=" . $this->szTransactionDateTime .
                "&CallbackURL=" . $this->szCallbackURL .
                "&OrderDescription=" . $this->szOrderDescription .
                "&CustomerName=" . $this->szCardName .
                "&Address1=" . $this->szAddress1 .
                "&Address2=" . $this->szAddress2 .
                "&Address3=" . $this->szAddress3 .
                "&Address4=" . $this->szAddress4 .
                "&City=" . $this->szCity .
                "&State=" . $this->szState .
                "&PostCode=" . $this->szPostCode .
                "&CountryCode=" . $this->iccISOCountryCode .
                "&EmailAddress=" . $this->szEmailAddress .
                "&PhoneNumber=" . $this->szPhoneNumber .
                "&EmailAddressEditable=" . self::boolToString($this->boEmailAddressEditable) .
                "&PhoneNumberEditable=" . self::boolToString($this->boPhoneNumberEditable) .
                "&CV2Mandatory=" . self::boolToString($this->boCV2Mandatory) .
                "&Address1Mandatory=" . self::boolToString($this->boAddress1Mandatory) .
                "&CityMandatory=" . self::boolToString($this->boCityMandatory) .
                "&PostCodeMandatory=" . self::boolToString($this->boPostCodeMandatory) .
                "&StateMandatory=" . self::boolToString($this->boStateMandatory) .
                "&CountryMandatory=" . self::boolToString($this->boCountryMandatory) .
                "&ResultDeliveryMethod=" . $this->rdmResultDeliveryMethod .
                "&ServerResultURL=" . $this->szServerResultURL .
                "&PaymentFormDisplaysResult=" . self::boolToString($this->boPaymentFormDisplaysResult) .
                "&ServerResultURLCookieVariables=" .
                "&ServerResultURLFormVariables=" .
                "&ServerResultURLQueryStringVariables=";

        return $szReturnString;
    }

    private function IncludePreSharedKeyInString() {

        switch ($this->hmHashMethod) {
            case HashMethod::MD5 :
                $boIncludePreSharedKeyInString = true;
                break;
            case HashMethod::SHA1 :
                $boIncludePreSharedKeyInString = true;
                break;
            case HashMethod::HMACMD5:
                $boIncludePreSharedKeyInString = false;
                break;
            case HashMethod::HMACSHA1:
                $boIncludePreSharedKeyInString = false;
                break;
        }

        return $boIncludePreSharedKeyInString;
    }

    private function generateStringToHash2() {

        $szReturnString = "";

        if ($this->IncludePreSharedKeyInString()) {
            $szReturnString = "PreSharedKey=" . $this->szPreSharedKey . "&";
        }

        $szReturnString .=
                "MerchantID=" . $this->szMerchantID .
                "&Password=" . $this->szPassword .
                "&StatusCode=" . $this->trTransactionResult->getStatusCode() .
                "&Message=" . $this->trTransactionResult->getMessage() .
                "&PreviousStatusCode=" . $this->trTransactionResult->getPreviousStatusCode() .
                "&PreviousMessage=" . $this->trTransactionResult->getPreviousMessage() .
                "&CrossReference=" . $this->trTransactionResult->getCrossReference();

        if ($this->rdmResultDeliveryMethod == ResultDeliveryMethod::POST) {
            $szReturnString .=
                    "&AddressNumericCheckResult=" . $this->trTransactionResult->getAddressNumericCheckResult() .
                    "&PostCodeCheckResult=" . $this->trTransactionResult->getPostCodeCheckResult() .
                    "&CV2CheckResult=" . $this->trTransactionResult->getCV2CheckResult() .
                    "&ThreeDSecureCheckResult=" . $this->trTransactionResult->getThreeDSecureCheckResult() .
                    "&CardType=" . $this->trTransactionResult->getCardType() .
                    "&CardClass=" . $this->trTransactionResult->getCardClass() .
                    "&CardIssuer=" . $this->trTransactionResult->getCardIssuer() .
                    "&CardIssuerCountryCode=" . $this->trTransactionResult->getCardIssuerCountryCode();
        }

        $szReturnString .=
                "&Amount=" . $this->trTransactionResult->getAmount() .
                "&CurrencyCode=" . $this->trTransactionResult->getCurrencyCode() .
                "&OrderID=" . $this->trTransactionResult->getOrderID() .
                "&TransactionType=" . $this->trTransactionResult->getTransactionType() .
                "&TransactionDateTime=" . $this->trTransactionResult->getTransactionDateTime() .
                "&OrderDescription=" . $this->trTransactionResult->getOrderDescription() .
                "&CustomerName=" . $this->trTransactionResult->getCustomerName() .
                "&Address1=" . $this->trTransactionResult->getAddress1() .
                "&Address2=" . $this->trTransactionResult->getAddress2() .
                "&Address3=" . $this->trTransactionResult->getAddress3() .
                "&Address4=" . $this->trTransactionResult->getAddress4() .
                "&City=" . $this->trTransactionResult->getCity() .
                "&State=" . $this->trTransactionResult->getState() .
                "&PostCode=" . $this->trTransactionResult->getPostCode() .
                "&CountryCode=" . $this->trTransactionResult->getCountryCode();

        if ($this->rdmResultDeliveryMethod == ResultDeliveryMethod::POST) {
            $szReturnString .=
                    "&EmailAddress=" . $this->trTransactionResult->getEmailAddress() .
                    "&PhoneNumber=" . $this->trTransactionResult->getPhoneNumber();
        }

        return ($szReturnString);
    }

    private function generateStringToHash3() {

        $szReturnString = "";

        if ($this->IncludePreSharedKeyInString()) {
            $szReturnString = "PreSharedKey=" . $this->szPreSharedKey . "&";
        }

        $szReturnString .=
                "MerchantID=" . $this->szMerchantID .
                "&Password=" . $this->szPassword .
                "&CrossReference=" . $this->szHostedCrossReference .
                "&OrderID=" . $this->szOrderID;

        return ($szReturnString);
    }

    private function calculateHashDigest() {

        $szHashDigest = null;
        switch ($this->hmHashMethod) {
            case HashMethod::MD5 :
                $szHashDigest = md5($this->szStringToHash);
                break;
            case HashMethod::SHA1:
                $szHashDigest = sha1($this->szStringToHash);
                break;
            case HashMethod::HMACMD5:
                $szHashDigest = hash_hmac("md5", $this->szStringToHash, $this->szPreSharedKey);
                break;
            case HashMethod::HMACSHA1:
                $szHashDigest = hash_hmac("sha1", $this->szStringToHash, $this->szPreSharedKey);
                break;
        }

        return $szHashDigest;
    }

    private function validateTransactionResult_POST() {

        $this->boErrorActive = false;

        // read the transaction result variables from the post variable list
        if (!$this->getTransactionResultFromPostVariables()) {
            $this->boErrorActive  = true;
            $this->szErrorMessage = $this->szErrorMessage;
        } else {
            // now need to validate the hash digest
            $this->szStringToHash = $this->generateStringToHash2();
            $this->szHashDigest   = $this->calculateHashDigest();

            // does the calculated hash match the one that was passed?
            if (strToUpper($this->aHostedTransactionResponse['HashDigest']) != strToUpper($this->szHashDigest)) {
                $this->boErrorActive  = true;
                $this->szErrorMessage = "Hash digests don't match - possible variable tampering";
            }
        }

        return (!$this->boErrorActive);
    }

    private function validateTransactionResult_SERVER() {

        $this->boErrorActive = false;

        // read the transaction reference variables from the query string variable list
        if ($this->hfmHostedFormMethod == HostedFormMethod::ReceiveResults) {
            if ($this->getTransactionResultFromPostVariables()) {
                $this->szStringToHash = $this->generateStringToHash2();
            }
        } else {
            if ($this->getTransactionReferenceFromQueryString()) {
                $this->szStringToHash = $this->generateStringToHash3();
            }
        }

        if (!$this->boErrorActive) {

            // now need to validate the hash digest
            $szCalculatedHashDigest = $this->calculateHashDigest();

            // does the calculated hash match the one that was passed?
            if (strToUpper($szCalculatedHashDigest) != strToUpper($this->szHashDigest)) {
                $this->boErrorActive  = true;
                $this->szErrorMessage = "Hash digests don't match - possible variable tampering";
                $this->szErrorMessage .= "\n\n" . $this->szStringToHash . "\n" . $this->szHashDigest . "\n\n" . $this->aHostedTransactionResponse['HashDigest'] . "\n\n" . print_r($this->aHostedTransactionResponse, 1);
            } else {

                require 'Hosted/ServerMethodResponse.php';

                if ($this->hfmHostedFormMethod == HostedFormMethod::ReceiveResults) {//&& $this->boDatabaseSupport ) {
                    StoreServerMethodResponse($this->aHostedTransactionResponse);
                } else {

                    $this->aHostedTransactionResponse = RetrieveServerMethodResponse($this->szHostedCrossReference);

                    $this->trTransactionResult = new TransactionResult();
                    $this->trTransactionResult->setStatusCode($this->aHostedTransactionResponse['StatusCode']); // transaction status code
                    $this->trTransactionResult->setMessage($this->aHostedTransactionResponse['Message']); // transaction message
                    $this->trTransactionResult->setPreviousStatusCode($this->aHostedTransactionResponse['PreviousStatusCode']); // status code of original transaction if duplicate transaction
                    $this->trTransactionResult->setPreviousMessage($this->aHostedTransactionResponse['PreviousMessage']); // status code of original transaction if duplicate transaction
                    $this->trTransactionResult->setCrossReference($this->aHostedTransactionResponse['CrossReference']); // cross reference of transaction
                    $this->trTransactionResult->setAddressNumericCheckResult($this->aHostedTransactionResponse['AddressNumericCheckResult']);
                    $this->trTransactionResult->setPostCodeCheckResult($this->aHostedTransactionResponse['PostCodeCheckResult']);
                    $this->trTransactionResult->setCV2CheckResult($this->aHostedTransactionResponse['CV2CheckResult']);
                    $this->trTransactionResult->setThreeDSecureCheckResult($this->aHostedTransactionResponse['ThreeDSecureCheckResult']);
                    $this->trTransactionResult->setCardType($this->aHostedTransactionResponse['CardType']);
                    $this->trTransactionResult->setCardClass($this->aHostedTransactionResponse['CardClass']);
                    $this->trTransactionResult->setCardIssuer($this->aHostedTransactionResponse['CardIssuer']);
                    $this->trTransactionResult->setCardIssuerCountryCode($this->aHostedTransactionResponse['CardIssuerCountryCode']);
                    $this->trTransactionResult->setAmount($this->aHostedTransactionResponse['Amount']); // amount echoed back
                    $this->trTransactionResult->setCurrencyCode($this->aHostedTransactionResponse['CurrencyCode']); // currency code echoed back
                    $this->trTransactionResult->setOrderID($this->aHostedTransactionResponse['OrderID']); // order ID echoed back
                    $this->trTransactionResult->setTransactionType($this->aHostedTransactionResponse['TransactionType']); // transaction type echoed back
                    $this->trTransactionResult->setTransactionDateTime($this->aHostedTransactionResponse['TransactionDateTime']); // transaction date/time echoed back
                    $this->trTransactionResult->setOrderDescription($this->aHostedTransactionResponse['OrderDescription']); // order description echoed back
                    // the customer details that were actually
                    // processed (might be different
                    // from those passed to the payment form)
                    $this->trTransactionResult->setCustomerName($this->aHostedTransactionResponse['CustomerName']);
                    $this->trTransactionResult->setAddress1($this->aHostedTransactionResponse['Address1']);
                    $this->trTransactionResult->setAddress2($this->aHostedTransactionResponse['Address2']);
                    $this->trTransactionResult->setAddress3($this->aHostedTransactionResponse['Address3']);
                    $this->trTransactionResult->setAddress4($this->aHostedTransactionResponse['Address3']);
                    $this->trTransactionResult->setCity($this->aHostedTransactionResponse['City']);
                    $this->trTransactionResult->setState($this->aHostedTransactionResponse['State']);
                    $this->trTransactionResult->setPostCode($this->aHostedTransactionResponse['PostCode']);
                    $this->trTransactionResult->setCountryCode($this->aHostedTransactionResponse['CountryCode']);
                    $this->trTransactionResult->setEmailAddress($this->aHostedTransactionResponse['EmailAddress']);
                    $this->trTransactionResult->setPhoneNumber($this->aHostedTransactionResponse['PhoneNumber']);



                    $this->setResultsAmountAndCurrency($this->trTransactionResult->getAmount(), $this->trTransactionResult->getCurrencyCode());
                    $this->ttTransactionType  = $this->trTransactionResult->getTransactionType();
                    $this->szOrderID          = $this->trTransactionResult->getOrderID();
                    $this->szOrderDescription = $this->trTransactionResult->getOrderDescription();
                }
            }
        }

        return (!$this->boErrorActive);
    }

    private function validateTransactionResult_SERVER_PULL() {

        $this->boErrorActive       = false;
        $this->trTransactionResult = null;

        // read the transaction reference variables from the query string variable list
        if (!$this->getTransactionReferenceFromQueryString()) {
            $this->boErrorActive = true;
        } else {
            // now need to validate the hash digest
            $this->szStringToHash   = $this->generateStringToHash3();
            $szCalculatedHashDigest = $this->calculateHashDigest();

            // does the calculated hash match the one that was passed?
            if (strToUpper($this->szHashDigest) != strToUpper($szCalculatedHashDigest)) {
                $this->boErrorActive  = true;
                $this->szErrorMessage = "Hash digests don't match - possible variable tampering";
            } else {
                // use the cross reference and/or the order ID to pull the
                // transaction results out of storage
                if (!$this->getTransactionResultFromPaymentFormHandler()) {
                    $this->szErrorMessage = "Error querying transaction result [" . $this->szCrossReference . "] from [" . $szPaymentFormResultHandlerURL . "]: " . $this->szErrorMessage;
                    $this->boErrorActive  = true;
                } else {
                    $this->boErrorActive = false;
                }
            }
        }

        return (!$this->boErrorActive);
    }

    private function getTransactionResultFromPaymentFormHandler() {

        $this->boErrorActive       = false;
        $this->szErrorMessage      = "";
        $this->trTransactionResult = null;

        try {
            // use curl to post the cross reference to the
            // payment form to query its status
            $cCURL = curl_init();

            // build up the post string
            $szPostString = "MerchantID=" . urlencode($this->szMerchantID) . "&Password=" . urlencode($this->szPassword) . "&CrossReference=" . urlencode($this->szHostedCrossReference);

            curl_setopt($cCURL, CURLOPT_URL, $this->getHostedPaymentHandler());
            curl_setopt($cCURL, CURLOPT_POST, true);
            curl_setopt($cCURL, CURLOPT_POSTFIELDS, $szPostString);
            curl_setopt($cCURL, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($cCURL, CURLOPT_RETURNTRANSFER, true);

            // read the response
            $szResponse   = curl_exec($cCURL);
            $szErrorNo    = curl_errno($cCURL);
            $ezErrorMsg   = curl_error($cCURL);
            $szHeaderInfo = curl_getinfo($cCURL);

            curl_close($cCURL);

            if ($szResponse == "") {
                $this->boErrorActive  = true;
                $this->szErrorMessage = "Received empty response from payment form hander";
            } else {
                try {
                    // parse the response into an array
                    $this->aHostedTransactionResponse = PaymentFormHelper::parseNameValueStringIntoArray($szResponse, true);

                    if (!isset($this->aHostedTransactionResponse["StatusCode"]) OR
                            intval($this->aHostedTransactionResponse["StatusCode"]) != 0) {
                        $this->boErrorActive = true;

                        // the message field is expected if the status code is non-zero
                        if (!isset($this->aHostedTransactionResponse["Message"])) {
                            $this->szErrorMessage = "Received invalid response from payment form hander [" . $szResponse . "]";
                        } else {
                            $this->szErrorMessage = $this->aHostedTransactionResponse["Message"];
                        }
                    } else {
                        // status code is 0, so	get the transaction result
                        if (!isset($this->aHostedTransactionResponse["TransactionResult"])) {
                            $this->boErrorActive  = true;
                            $this->szErrorMessage = "No transaction result in response from payment form hander [" . $szResponse . "]";
                        } else {
                            // parse the URL decoded transaction result field into a name value array
                            $aTransactionResultArray          = PaymentFormHelper::parseNameValueStringIntoArray(urldecode($this->aHostedTransactionResponse["TransactionResult"]), false);
                            $this->aHostedTransactionResponse = $aTransactionResultArray;

                            // parse this array into a transaction result object
                            if (!$this->getTransactionResultFromPostVariables()) {
                                $this->boErrorActive  = true;
                                $this->szErrorMessage = "Error [" . $this->szErrorMessage . "] parsing transaction result [" . urldecode($this->aHostedTransactionResponse["TransactionResult"]) . "] in response from payment form hander [" . $szResponse . "]";

                                if ($this->boErrorActive && $this->boDebugMode && $this->szDebugEmail != NULL) {
                                    mail($this->szDebugEmail, "FUNC: IridiumHosted:getTransactionResultFromPostVariables()", "ERROR: \n" . print_r($this->szErrorMessage, 1));
                                }
                            } else {
                                $this->boErrorActive = false;
                            }
                        }
                    }
                } catch (Exception $e) {
                    $this->boErrorActive  = true;
                    $this->szErrorMessage = "Exception [" . $e->getMessage() . "] when processing response from payment form handler [" . $szResponse . "]";
                }
            }
        } catch (Exception $e) {
            $this->boErrorActive  = true;
            $this->szErrorMessage = $e->getMessage();
        }

        return (!$this->boErrorActive);
    }

    private function IsReady() {

        parent::IsReadyBase();

        if ($this->hmHashMethod == HashMethod::NONE) {
            array_push($this->szErrorMessage, Errors::NoHashMethodSelected);
        }
        if ($this->szPreSharedKey == NULL) {
            array_push($this->szErrorMessage, Errors::NoPreSharedKeySpecified);
        }

        if ($this->hfmHostedFormMethod == HostedFormMethod::NONE) {
            array_push($this->szErrorMessage, Errors::NoHostedFormMethodSelected);
        } else {
            switch ($this->hfmHostedFormMethod)
            {
                case HostedFormMethod::Process:
                    if ($this->ttTransactionType == TransactionType::NONE) {
                        array_push($this->szErrorMessage, Errors::NoTransactionTypeSelected);
                    }
                    if ($this->nAmount == NULL) {
                        array_push($this->szErrorMessage, Errors::NoAmountSpecified);
                    }
                    if ($this->iccISOCurrencyCode == NULL) {
                        array_push($this->szErrorMessage, Errors::NoCurrencySpecified);
                    }
                    if ($this->rdmResultDeliveryMethod == NULL) {
                        array_push($this->szErrorMessage, Errors::NoResultDeliveryMethodSelected);
                    }
                    if ($this->szCallbackURL == NULL) {
                        array_push($this->szErrorMessage, Errors::NoCallbackURLSpecified);
                    }
                    break;
                case HostedFormMethod::ReceiveResults:
                    if ($this->aHostedTransactionResponse == null) {
                    array_push($this->szErrorMessage, Errors::NoHostedTransactionResponseSpecified);
                    }
                    break;
                case HostedFormMethod::DisplayResults:
                    
                    break;
            }
        }

        $this->boErrorActive = !(empty($this->szErrorMessage));

        if (!$this->boErrorActive) {
            if ($this->szHTML_ButtonCaption == NULL) {
                $this->szHTML_ButtonCaption = "Pay Now";
            }
        }

        if ($this->boErrorActive && $this->boDebugMode && $this->szDebugEmail != NULL) {
            mail($this->szDebugEmail, "FUNC: IridiumHosted:IsReady()", "ERROR: \n" . print_r($this->szErrorMessage, 1));
        }
        return !$this->boErrorActive;
    }

    public function Process() {

        $return = NULL;

        if ($this->IsReady()) {
            parent::Process();

            switch ($this->hfmHostedFormMethod) {
                case HostedFormMethod::Process:
                    switch ($this->hfrtHostedFormReturnType) {
                        case HostedFormReturnType::HTMLFormCode:
                            $return = $this->GenerateHostedPaymentForm();
                            break;
                    }
                    break;
                default:
                    $return = $this->HostedTransactionResult();
                    break;
            }

            //            if (!$this->boTransactionProcessed) {
            //                $this->boErrorActive = TRUE;
            //                //array_push($this->szErrorMessage, Errors::NoCommunicationWithGateway);
            //            }
        } else {
            $return = FALSE;
        }

        return $return;
    }

    private function GenerateHostedPaymentForm() {

        $this->szStringToHash = $this->generateStringToHash();
        $this->szHashDigest   = $this->calculateHashDigest();

        if ($this->boIncludeFormTags) {
            $return = '<form name="contactForm" id="contactForm" method="POST" action="https://mms.iridiumcorp.net/Pages/PublicPages/PaymentForm.aspx" >';
        }
        $return .=
                '
                                        <input type="hidden" name="HashDigest" value="' . $this->szHashDigest . '" />
					<input type="hidden" name="MerchantID" value="' . $this->szMerchantID . '" />
					<input type="hidden" name="Amount" value="' . $this->nAmountUndecimalised . '" />
					<input type="hidden" name="CurrencyCode" value="' . $this->iccISOCurrencyCode . '" />
                                    ' .
                '
					<input type="hidden" name="EchoAVSCheckResult" value="' . $this->boolToString($this->boEchoAVSCheckResult) . '" />
					<input type="hidden" name="EchoCV2CheckResult" value="' . $this->boolToString($this->boEchoCV2CheckResult) . '" />
					<input type="hidden" name="EchoThreeDSecureAuthenticationCheckResult" value="' . $this->boolToString($this->boEchoThreeDSecureAuthenticationCheckResult) . '" />
					<input type="hidden" name="EchoCardType" value="' . $this->boolToString($this->boEchoCardType) . '" />
                                    ' .
                //                /* <input type="hidden" name="AVSOverridePolicy" value="" />
                //                  <input type="hidden" name="CV2OverridePolicy" value="" />
                //                 */
                //                '

                '
					<input type="hidden" name="ThreeDSecureOverridePolicy" value="' . $this->boolToString($this->boThreeDSecureOverridePolicy) . '" />
                                    ' .
                '
					<input type="hidden" name="OrderID" value="' . $this->szOrderID . '" />
					<input type="hidden" name="TransactionType" value="' . $this->ttTransactionType . '" />
					<input type="hidden" name="TransactionDateTime" value="' . $this->szTransactionDateTime . '" />
					<input type="hidden" name="CallbackURL" value="' . $this->szCallbackURL . '" />
                                    ' .
                '
					<input type="hidden" name="OrderDescription" value="' . $this->szOrderDescription . '" />
					<input type="hidden" name="CustomerName" value="' . $this->szCardName . '" />
					<input type="hidden" name="Address1" value="' . $this->szAddress1 . '" />
					<input type="hidden" name="Address2" value="' . $this->szAddress2 . '" />
					<input type="hidden" name="Address3" value="' . $this->szAddress3 . '" />
					<input type="hidden" name="Address4" value="' . $this->szAddress4 . '" />
					<input type="hidden" name="City" value="' . $this->szCity . '" />
					<input type="hidden" name="State" value="' . $this->szState . '" />
					<input type="hidden" name="PostCode" value="' . $this->szPostCode . '" />
					<input type="hidden" name="CountryCode" value="' . $this->iccISOCountryCode . '" />
                                    ' .
                '
					<input type="hidden" name="EmailAddress" value="' . $this->szEmailAddress . '" />
					<input type="hidden" name="PhoneNumber" value="' . $this->szPhoneNumber . '" />
					<input type="hidden" name="EmailAddressEditable" value="' . $this->boolToString($this->boEmailAddressEditable) . '" />
					<input type="hidden" name="PhoneNumberEditable" value="' . $this->boolToString($this->boPhoneNumberEditable) . '" />
                                    ' .
                '
					<input type="hidden" name="CV2Mandatory" value="' . $this->boolToString($this->boCV2Mandatory) . '" />
					<input type="hidden" name="Address1Mandatory" value="' . $this->boolToString($this->boAddress1Mandatory) . '" />
					<input type="hidden" name="CityMandatory" value="' . $this->boolToString($this->boCityMandatory) . '" />
					<input type="hidden" name="PostCodeMandatory" value="' . $this->boolToString($this->boPostCodeMandatory) . '" />
					<input type="hidden" name="StateMandatory" value="' . $this->boolToString($this->boStateMandatory) . '" />
					<input type="hidden" name="CountryMandatory" value="' . $this->boolToString($this->boCountryMandatory) . '" />
					<input type="hidden" name="ResultDeliveryMethod" value="' . $this->rdmResultDeliveryMethod . '" />
					<input type="hidden" name="ServerResultURL" value="' . $this->szServerResultURL . '" />
					<input type="hidden" name="PaymentFormDisplaysResult" value="' . $this->boolToString($this->boPaymentFormDisplaysResult) . '" />
                                    ' .
                '
					<input type="hidden" name="ServerResultURLCookieVariables" value="" />
					<input type="hidden" name="ServerResultURLFormVariables" value="" />
					<input type="hidden" name="ServerResultURLQueryStringVariables" value="" />
                                    ';

        if ($this->boIncludeFormSubmitButton) {
            $return .= '<input type="submit" value="' . $this->szHTML_ButtonCaption . '" />';
        }

        if ($this->boIncludeFormTags) {
            $return .= '</form>';
        }
        return $return;
    }

    private function HostedTransactionResult() {

        switch ($this->rdmResultDeliveryMethod) {
            case "POST":

                $this->boTransactionProcessed = $this->validateTransactionResult_POST();

                if ($this->boTransactionProcessed) {
                    $this->setResultsAmountAndCurrency($this->trTransactionResult->getAmount(), $this->trTransactionResult->getCurrencyCode());
                    $this->ttTransactionType  = $this->trTransactionResult->getTransactionType();
                    $this->szOrderID          = $this->trTransactionResult->getOrderID();
                    $this->szOrderDescription = $this->trTransactionResult->getOrderDescription();
                }

                break;
            case "SERVER":

                // the results have already been delivered via a server-to-server
                // call from the payment form to the ServerResultURL
                // need to query these transaction results to display
                $this->boTransactionProcessed = $this->validateTransactionResult_SERVER();

                break;
            case "SERVER_PULL":

                // need to query the results from the payment form using the passed
                // cross reference

                $this->boTransactionProcessed = $this->validateTransactionResult_SERVER_PULL();
                // the results need to be stored here as this is the first time
                // they will have touched this system
                if ($this->boTransactionProcessed) {

                    $this->setResultsAmountAndCurrency($this->trTransactionResult->getAmount(), $this->trTransactionResult->getCurrencyCode());
                    $this->ttTransactionType  = $this->trTransactionResult->getTransactionType();
                    $this->szOrderID          = $this->trTransactionResult->getOrderID();
                    $this->szOrderDescription = $this->trTransactionResult->getOrderDescription();

                    if (!$this->boTransactionProcessed) {
                        // handle the case where the results aren't stored correctly
                    }
                }
                break;
        }

        return $this->boTransactionProcessed;
    }

}
abstract class IridiumSQL {
    
    const tblGEP_EntryPoints    = "tblIridium_GEP_EntryPoints";
    const tblHPF_Transactions   = "tblIridium_HPF_Transactions";
    const tbl3DS_Transactions   = "tblIridium_3DS_Transactions";
    const tblHPF_SERVER_Results = "tblIridium_HPF_SERVER_Results";
        
    public static $g_szQueryString;
    
    public function __constructor() {

    }
    
    public static function TableExists($szTableName){
        self::$g_szQueryString = 
            "   SHOW TABLES LIKE '$szTableName'";
        return self::$g_szQueryString;
    }
    public static function createGEP_EntryPoints() {

        self::$g_szQueryString = 
            "   CREATE TABLE `" . self::tblGEP_EntryPoints . "`
                    (
                        `ID`                    int(11)     NOT NULL AUTO_INCREMENT,
                        `GatewayEntryPoint`     text(255)   NOT NULL,
                        `TransactionDateTime`   DateTime    NOT NULL,
                        PRIMARY KEY (`ID`)
                    );";

        return self::$g_szQueryString;
    }
    
    public static function insertGEP_EntryPoint($GatewayEntryPointURL, $szTransactionDateTime) {

        self::$g_szQueryString = 
            "   INSERT INTO " . self::tblGEP_EntryPoints . "
                (
                    `GatewayEntryPoint`,
                    `TransactionDateTime`
                )
                VALUES
                (
                    '$GatewayEntryPointURL',
                    '$szTransactionDateTime'
                )";
        return self::$g_szQueryString;
    }
    public static function selectGEP_EntryPoint($RemoveDataThreshold_AmountInMinutes) {

        self::$g_szQueryString = 
            "   SELECT `GatewayEntryPoint`, MAX(`TransactionDateTime`)
                FROM " . self::tblGEP_EntryPoints . "
                WHERE `TransactionDateTime` >= SUBDATE(NOW(), INTERVAL $RemoveDataThreshold_AmountInMinutes MINUTE)";

        return self::$g_szQueryString;
    }

    public static function deleteGEP_EntryPoint($RemoveDataThreshold_AmountInMinutes) {

        self::$g_szQueryString = 
            "   DELETE FROM " . self::tblGEP_EntryPoints . "
                WHERE `TransactionDateTime` < SUBDATE(NOW(), INTERVAL $RemoveDataThreshold_AmountInMinutes MINUTE)";

        return self::$g_szQueryString;
    }

    public static function create3DS_Transactions() {

        self::$g_szQueryString = 
            "   CREATE TABLE `" . self::tbl3DS_Transactions . "`
                (
                    `CrossReference`        varchar(24) NOT NULL,
                    `UserID`                int(10)     NOT NULL,
                    `ISOCurrencyCode`       text(3)     NOT NULL,
                    `Amount`                varchar(12) NOT NULL,
                    `TransactionDateTime`   DateTime    NOT NULL,
                    PRIMARY KEY (`CrossReference`)
                );";

        return self::$g_szQueryString;
    }

    public static function insert3DS_Transaction($szCrossReference, $szUserID, $iccISOCurrencyCode, $nAmount, $szTransactionDateTime) {

        self::$g_szQueryString = 
            "   INSERT INTO " . self::tbl3DS_Transactions . "
                (
                    `CrossReference`,
                    `UserID`,
                    `ISOCurrencyCode`,
                    `Amount`,
                    `TransactionDateTime`
                )
                VALUES
                (
                    '$szCrossReference',
                    '$szUserID',
                    '$iccISOCurrencyCode',
                    '$nAmount',
                    '$szTransactionDateTime'
                )";

        return self::$g_szQueryString;
    }

    public static function select3DS_UserID($CrossReference) {

        self::$g_szQueryString = 
            "   SELECT `UserID`
                FROM " . self::tbl3DS_Transactions . "
                WHERE `CrossReference` = '$CrossReference'";

        return self::$g_szQueryString;
    }

    public static function select3DS_ISOCurrencyCode($CrossReference) {

        self::$g_szQueryString = 
            "   SELECT `ISOCurrencyCode`
                FROM " . self::tbl3DS_Transactions . "
                WHERE `CrossReference` = '$CrossReference'";

        return self::$g_szQueryString;
    }

    public static function select3DS_Amount($CrossReference) {

        self::$g_szQueryString = 
            "   SELECT `Amount`
                FROM " . self::tbl3DS_Transactions . "
                WHERE `CrossReference` = '$CrossReference'";

        return self::$g_szQueryString;
    }

    public static function delete3DS_Transaction($CrossReference, $UserID) {

        self::$g_szQueryString = 
            "   DELETE FROM " . self::tbl3DS_Transactions . "
                WHERE `CrossReference` = '$CrossReference'
                AND `UserID` = '$UserID'";

        return self::$g_szQueryString;
    }

    public static function delete3DS_HistoricTransactions() {

        self::$g_szQueryString = 
            "   DELETE FROM " . self::tbl3DS_Transactions . "
                WHERE `TransactionDateTime` < SUBDATE(NOW(), INTERVAL 2 DAY)";

        return self::$g_szQueryString;
    }

    public static function createHPF_RESULTS() {

        self::$g_szQueryString = 
            "   CREATE TABLE `" . self::tblHPF_SERVER_Results . "`
                (
                    `HashDigest`                text(64)    NOT NULL,
                    `MerchantID`                text(15)    NOT NULL,
                    `StatusCode`                text(3)     NOT NULL,
                    `Message`                   text(512)   ,
                    `PreviousStatusCode`        text(3)     ,
                    `PreviousMessage`           text(512)   ,
                    `CrossReference`            varchar(24) NOT NULL,
                    `Amount`                    text(13)    NOT NULL,
                    `CurrencyCode`              text(3)     NOT NULL,
                    `OrderID`                   text(50)    NOT NULL,
                    `TransactionType`           text(50)    NOT NULL,
                    `TransactionDateTime`       DateTime    NOT NULL,
                    `OrderDescription`          text(256)   ,
                    `CustomerName`              text(100)   ,
                    `Address1`                  text(100)   ,
                    `Address2`                  text(100)   ,
                    `Address3`                  text(100)   ,
                    `Address4`                  text(100)   ,
                    `City`                      text(100)   ,
                    `State`                     text(100)   ,
                    `PostCode`                  text(100)   ,
                    `CountryCode`               text(3)     ,
                    `EmailAddress`              text(256)   ,
                    `PhoneNumber`               text(50)    ,
                    `CardType`                  text(100)   ,
                    `CardClass`                 text(100)   ,
                    `CardIssuer`                text(256)   ,
                    `CardIssuerCountryCode`     text(3)     ,
                    `AddressNumericCheckResult` text(50)    ,
                    `PostCodeCheckResult`       text(50)    ,
                    `CV2CheckResult`            text(50)    ,
                    `ThreeDSecureCheckResult`   text(50)    ,

                    PRIMARY KEY (`CrossReference`)
                );";

        return self::$g_szQueryString;
    }

    public static function insertHPF_SERVER_Results($aResponseVariables) {

        self::$g_szQueryString =
            "   INSERT INTO " . self::tblHPF_SERVER_Results . "
                (";

                foreach ($aResponseVariables as $key => $value) {
                    self::$g_szQueryString .=
                            "
                                `" . $key . "`,";
                }

                self::$g_szQueryString = substr(self::$g_szQueryString, 0, strlen(self::$g_szQueryString) - 1);
    
                self::$g_szQueryString .=
                        "
                            )
                            VALUES
                            (
                            ";
    
                foreach ($aResponseVariables as $key => $value) {
                    self::$g_szQueryString .= "
                                '" . $value . "',";
                }
                self::$g_szQueryString = substr(self::$g_szQueryString, 0, strlen(self::$g_szQueryString) - 1);

                self::$g_szQueryString .=
            "   );";

        return self::$g_szQueryString;
    }

    public static function selectHPF_SERVER_Results($CrossReference) {

        $results = array();

        self::$g_szQueryString = 
            "   SELECT *
                FROM " . self::tblHPF_SERVER_Results . "
                WHERE `CrossReference` = '$CrossReference'";

        return self::$g_szQueryString;
    }

    public static function deleteHPF_HistoricResults() {

        self::$g_szQueryString =
                "   DELETE FROM " . self::tblHPF_SERVER_Results . "
                    WHERE `TransactionDateTime` < SUBDATE(NOW(), INTERVAL 2 DAY)";

        return self::$g_szQueryString;
    }
}


?>